@@ -208,15 +208,6 @@ impl<'de> Deserializer<'de> {
     where
         V: serde::de::Visitor<'de>,
     {
-        if let DeserializerHint::BinarySubtype(expected_st) = hint {
-            if self.current_type != ElementType::Binary {
-                return Err(Error::custom(format!(
-                    "expected Binary with subtype {:?}, instead got {:?}",
-                    expected_st, self.current_type
-                )));
-            }
-        }
-
         match self.current_type {
             ElementType::Int32 => visitor.visit_i32(read_i32(&mut self.bytes)?),
             ElementType::Int64 => visitor.visit_i64(read_i64(&mut self.bytes)?),
@@ -1316,14 +1307,14 @@ impl<'de, 'a> CodeWithScopeDeserializer<'de, 'a> {
         F: FnOnce(&mut Self) -> Result<O>,
     {
         let start_bytes = self.root_deserializer.bytes.bytes_read();
-        let out = f(self);
+        let out = f(self)?;
         let bytes_read = self.root_deserializer.bytes.bytes_read() - start_bytes;
         self.length_remaining -= bytes_read as i32;
 
         if self.length_remaining < 0 {
             return Err(Error::custom("length of CodeWithScope too short"));
         }
-        out
+        Ok(out)
     }
 }
 
