@@ -181,6 +181,26 @@ impl RawDocument {
         Ok(None)
     }
 
+    /// Gets an iterator over the elements in the [`RawDocument`] that yields
+    /// `Result<(&str, RawBson<'_>)>`.
+    pub fn iter(&self) -> Iter<'_> {
+        Iter::new(self)
+    }
+
+    /// Gets an iterator over the elements in the [`RawDocument`],
+    /// which yields `Result<RawElement<'_>>` values. These hold a
+    /// reference to the underlying document but do not explicitly
+    /// resolve the values.
+    ///
+    /// This iterator, which underpins the implementation of the
+    /// default iterator, produces `RawElement` objects that hold a
+    /// view onto the document but do not parse out or construct
+    /// values until the `.value()` or `.try_into()` methods are
+    /// called.
+    pub fn iter_elements(&self) -> RawIter<'_> {
+        RawIter::new(self)
+    }
+
     fn get_with<'a, T>(
         &'a self,
         key: impl AsRef<str>,
@@ -599,6 +619,6 @@ impl<'a> IntoIterator for &'a RawDocument {
     type Item = Result<(&'a str, RawBsonRef<'a>)>;
 
     fn into_iter(self) -> Iter<'a> {
-        Iter::new(self)
+        self.iter()
     }
 }
